;												
; HC-SR04 Ultraschall fr ATmega8				
; Bernhard.Erfurt@gmx.de						
; 06/2020										
;												
; LCD 2x16 oder 4x16							
;												

;												
; LED ROT blinkt: Programm Check Error			
; LED ROT+GELB blinkt: Interrupt Error			
; LED ROT+GELB+GRN blinkt: WDR Error (nicht in dieser Version)	
;												

; ATmega8 										
.include "m8def.inc"
.include "MACRO.asm"
; KONSTANTEN									
.equ TAKT							=8000000	
; INPUT											
.equ INP_NR							= 2
.equ INP_DDR						= DDRD
.equ INP_PORT						= PORTD
.equ INP_PIN						= PIND
; LED-GRUEN										
.equ LED_GRUEN_PIN_NR				= 5
.equ LED_GRUEN_DDR					= DDRD
.equ LED_GRUEN_PORT					= PORTD
; LED-GELB										
.equ LED_GELB_PIN_NR				= 6
.equ LED_GELB_DDR					= DDRD
.equ LED_GELB_PORT					= PORTD
; LED-ROT										
.equ LED_ROT_PIN_NR					= 7
.equ LED_ROT_DDR					= DDRD
.equ LED_ROT_PORT					= PORTD
; LCD (DATA)									
.equ LCD_DATA0_DDR					= DDRC
.equ LCD_DATA1_DDR					= DDRC
.equ LCD_DATA2_DDR					= DDRC
.equ LCD_DATA3_DDR					= DDRC
.equ LCD_DATA0_PORT					= PORTC
.equ LCD_DATA1_PORT					= PORTC
.equ LCD_DATA2_PORT					= PORTC
.equ LCD_DATA3_PORT					= PORTC
.equ LCD_DATA0						= 2
.equ LCD_DATA1						= 3
.equ LCD_DATA2						= 4
.equ LCD_DATA3						= 5
; LCD RS (Befehl/DATEN)							
.equ LCD_RS_DDR						= DDRB
.equ LCD_RS_PORT					= PORTB
.equ LCD_RS							= 4
; LCD_ENABLE									
.equ LCD_EN_DDR						= DDRB
.equ LCD_EN_PORT					= PORTB
.equ LCD_EN							= 5
; SRAM-BELEGUNG ATmega8							
.equ adr_DATEN						=SRAM_START		; 0x0060 START Adresse 		

.equ adr_ZEIGER						=SRAM_START+9
.equ adr_MW							=SRAM_START+10

.equ adr_SRAM_CHECK					=SRAM_START+943	; SRAM CHECK muss immer "S" sein	

; REGISTER										
.def COUNT0	=R8
.def COUNT1	=R9

.def NULL 	= R12
.def EINS 	= R14
.def VOLL	= R15

.def temp 	= R16
.def temp1 	= R17
.def temp2 	= R18
.def temp3 	= R19
.def temp4 	= R20
.def temp5 	= R21
.def temp6 	= R22
.def temp7 	= R23
.def temp8 	= R24
.def temp9 	= R25


.cseg					;Beginn eines Code-Segmentes
.org 0					;Startadresse=0

rjmp RESET      		; Reset Handler                           				
rjmp EXT_INT0   		; IRQ0 Handler                      					
rjmp INTERRUPT_ERROR	;rjmp EXT_INT1   ; IRQ1 Handler                      	
rjmp INTERRUPT_ERROR	;rjmp TIM2_COMP  ; Timer2 Compare Handler            	
rjmp INTERRUPT_ERROR	;rjmp TIM2_OVF   ; Timer2 Overflow Handler           	
rjmp INTERRUPT_ERROR	;rjmp TIM1_CAPT  ; Timer1 Capture Handler            	
rjmp INTERRUPT_ERROR	;rjmp TIM1_COMPA ; Timer1 CompareA Handler           	
rjmp INTERRUPT_ERROR	;rjmp TIM1_COMPB ; Timer1 CompareB Handler           	
rjmp TIM1_OVF   		; Timer1 Overflow Handler           					
rjmp INTERRUPT_ERROR	;rjmp TIM0_OVF   ; Timer0 Overflow Handler           	
rjmp INTERRUPT_ERROR	;rjmp SPI_STC    ; SPI Transfer Complete Handler     	
rjmp INTERRUPT_ERROR	;rjmp ZEICHEN_EINLESEN 	; USART RX Complete Handler     
rjmp INTERRUPT_ERROR	;rjmp USART_UDRE ; UDR Empty Handler                 	
rjmp INTERRUPT_ERROR	;rjmp USART_TXC  ; USART TX Complete Handler         	
rjmp INTERRUPT_ERROR	;rjmp ADCi       ; ADC Conversion Complete Handler   	
rjmp INTERRUPT_ERROR	;rjmp EE_RDY     ; EEPROM Ready Handler              	
rjmp INTERRUPT_ERROR	;rjmp ANA_COMP   ; Analog Comparator Handler         	
rjmp INTERRUPT_ERROR	;rjmp TWSI       ; Two-wire Serial Interface Handler 	
rjmp INTERRUPT_ERROR	;rjmp SPM_RDY    ; Store Program Memory Ready Handler	
;-------------------------------------------------------------------------------
; Unkorrekten Interrupt Aufruf 													
INTERRUPT_ERROR:
	LED_GELB_ON
	LED_ROT_ON
	rcall wait_100ms
	LED_GRUEN_OFF
	LED_GELB_OFF
	LED_ROT_OFF
	rcall wait_100ms
	rjmp INTERRUPT_ERROR
;-------------------------------------------------------------------------------
RESET:                        
	ldi temp, LOW(RAMEND)					; STACK initialisieren								
    out SPL, temp
    ldi temp, HIGH(RAMEND)
    out SPH, temp

	rcall WDR_INITIALISIERUNG				; WDR_INITIALISIERUNG								
	rcall SRAM_CLEAR						; SRAM lschen										
	rcall PORT_CLEAR						; PORT_CLEAR										
	rcall REGISTER_CLEAR					; REGISTER-CLEAR									
	rcall STARTWERTE						; Startwerte										
	rcall LED_INITIALISIERUNG				; LED												
	rcall LCD_INITIALISIERUNG				; LCD												
	rcall ANZEIGE_BEGRUESSUNG				; LCD												
	rcall EXT_INTERRUPT_INITIALISIEREN		; ext.Interrupt										
	rcall TIMER1_INITIALISIERUNG			; Timer1											
	rcall TIMER2_CTC_INITIALISIERUNG		; Timer2											
	rcall PROGRAMM_CHECK					; CHECK												
	sei										; Interrupts freigeben
;-------------------------------------------------------------------------------
LOOP:
	LED_GELB_ON
	rcall WAIT_100us						; Pause	
	LED_GELB_OFF
;-------------------------------------------------------------------------------
LOOP_s:
	WDR										; Watch-Dog-RESET	
	tst COUNT0								; Messung fertig ?	
	brne LOOP_RUN
	tst COUNT1								; Messung fertig ?	
	brne LOOP_RUN
	rjmp LOOP_s
;-------------------------------------------------------------------------------
LOOP_RUN:
	rcall WAIT_10ms						; Pause			
	rcall MAX							; Maximalwert	
	rcall CALC_cm						; Umrechnung auf cm
	rcall ANZEIGE_ENTFERNUNG
	rcall PROGRAMM_CHECK
	clr COUNT0
	clr COUNT1
rjmp LOOP
; ##############################################################################
; ##############################################################################
; ##############################################################################
; Mittelwertberechnung															
; INP/OUT: COUNT0+COUNT1														
MAX:
	ldi ZL,low (adr_MW)		; Z initialisieren	
	ldi ZH,high(adr_MW)
	LDS temp,(adr_ZEIGER)	; Zeiger load		
	inc temp				; +1				
	andi temp,0b00000111	; begrenzen			
	STS(adr_ZEIGER),temp	; Zeiger save		

	LSL temp				; Zeiger x2			
	add ZL,temp				; Z + Zeiger		
	adc ZH,NULL
	ST Z+,COUNT0			; Data save			
	ST Z+,COUNT1
;-------------------------------------------------------------------------------
	ldi ZL,low (adr_MW)		; Z initialisieren	
	ldi ZH,high(adr_MW)
	ldi temp9,8				; Schleife			
MAX_s:
	LD R0,Z+				; Data load			
	LD R1,Z+	
	cp R1,COUNT1			; vergleich			
	breq MAX_L
	brlo MAX_E
	mov COUNT0,R0
	mov COUNT1,R1
	rjmp MAX_E
MAX_L:
	cp R0,COUNT0			; vergleich			
	brlo MAX_E
	mov COUNT0,R0
	mov COUNT1,R1
MAX_E:
	dec temp9				; Schleife			
	brne MAX_s
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
; Umrechnung auf cm																
; INP/OUT: COUNT0+COUNT1														
CALC_cm:
	mov temp1,COUNT0
	mov temp2,COUNT1
	ldi temp3,low (562)
	ldi temp4,high(562)
	rcall MULTIPLIKATION_16_16_32

	subi temp1, BYTE1(-1024/2)	; aufrunden
	sbci temp2, BYTE2(-1024/2)
	sbci temp3, BYTE3(-1024/2)
	sbci temp4, BYTE4(-1024/2)

	mov temp1,temp2				; :256
	mov temp2,temp3
	mov temp3,temp4
	LSR temp3					; :2
	ROR temp2
	ROR temp1
	LSR temp3					; :2
	ROR temp2
	ROR temp1

	mov COUNT0,temp1
	mov COUNT1,temp2
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
.include "EXT_INTERRUPT.asm"
.include "LCD_ANZEIGE.asm"
.include "LCD.asm"
.include "LCD_ASCII.asm"
.include "TIMER.asm"
.include "SONSTIGES.asm"
.include "WAIT.asm"
;-------------------------------------------------------------------------------
TEXT_BEGRUESSUNG_1:
.db "Ultraschall",0
TEXT_BEGRUESSUNG_2:
.db "HC-SR04",0
TEXT_BEGRUESSUNG_3:
.db "ATmega8",0
TEXT_BEGRUESSUNG_4:
.db "29.06.2020 ",0

